#!/usr/bin/env python
# -*- coding: utf-8 -*-
__project__ = 'ACAT'
__description__ = 'Alloy Catalysis Automated Toolkit'
__authors__ = ['Shuang Han']
__copyright__ = '2021'
__email__ = 'hanshuangshiren@gmail.com'
__license__ = 'GNU Lesser General Public License 3.0 (LGPL 3.0)'
__maintainer__ = 'Shuang Han'
__url__ = 'https://gitlab.com/asm-dtu/acat'
__version__ = '1.5.2'

